#!/usr/local/bin/python3.6


"""
Author :      Vincent <vincent.delft@gmail.com>
Version :     0.3
Licence :     BSD
Require :     python >= 3.6
              thinter
              xlip

Description : This tool allow you to display a calendar and copy
              the selected dates in your clipboard
              Works on openBSD and Windows 10
Changelog:    0.2 - initial public release
              0.3 - add root.geometry
           

/*
 * Copyright (c) 2018 Vincent Delft <vincent.delf@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
"""


import tkinter as tk
import calendar
import subprocess
import sys
import os


class MyCalendar:
    def __init__(self, root):
        self.root = root
        self.defaultcolor = None
        self.buttons = {}
        self.month = calendar.datetime.datetime.now().month
        self.year = calendar.datetime.datetime.now().year
        cal = calendar.Calendar()
        self.days = []
        self.day_names = ['','','','','','','']
        for day in cal.itermonthdays(self.year, self.month):
            self.days.append(day)
            if day:
                ddate = calendar.datetime.date(self.year, self.month, day)
                if ddate.strftime("%a") not in self.day_names:
                    pos = int(ddate.strftime("%u"))
                    self.day_names[pos-1] = ddate.strftime("%a")

    def click(self, pos):
        # print("pos", pos, self.buttons["%s%s" % (pos[0], pos[1])][0].cget('bg'))
        if self.buttons["%s%s" % (pos[0], pos[1])][0].cget('bg') != "tan1":
            if not self.defaultcolor:
                self.defaultcolor = self.buttons["%s%s" % (pos[0], pos[1])][0].cget('bg')
            self.buttons["%s%s" % (pos[0], pos[1])][0].configure(bg="tan1")
        else:
            self.buttons["%s%s" % (pos[0], pos[1])][0].configure(bg=self.defaultcolor)

    def export(self):
        text = ""
        selected = []
        for key in self.buttons.keys():
            # this loop must be done in correct sequence
            if self.buttons[key][0].cget('bg') == "tan1":
                text = "%s %s" % (text, str(self.buttons[key][1]))
                selected.append(self.buttons[key][1])
        selected.sort()
        if selected:
            prev = None
            between = []
            between_idx = -1
            for cur in selected:
                if prev and (cur.day - prev.day) == 1:
                    between[between_idx][1] = cur
                else:
                    between.append([cur, None])
                    between_idx += 1
                prev = cur
            textli = []
            for start, stop in between:
                if stop is None:
                    textli.append("on %s" % (start.strftime("%A %d")))
                else:
                    textli.append("between %s and %s" % (start.strftime("%A %d"), stop.strftime("%A %d")))
            text = ", ".join(textli) + " of %s" % start.strftime("%B")
            if os.name == "posix":
                subprocess.call("echo %s | xclip -selection clipboard" % text, shell=True)
            elif os.name == 'nt':
                subprocess.call("echo %s | clip" % text, shell=True)
            else:
                print("We cannot detect your OS. So we cannot put dates in your clipboard")
                sys.exit(1)

    def previous(self):
        self.month -= 1
        if self.month == 0:
            self.year -= 1
            self.month = 12
        cal = calendar.Calendar()
        self.days = []
        for day in cal.itermonthdays(self.year, self.month):
            self.days.append(day)
        self.display()

    def next(self):
        self.month += 1
        if self.month == 13:
            self.year += 1
            self.month = 1
        cal = calendar.Calendar()
        self.days = []
        for day in cal.itermonthdays(self.year, self.month):
            self.days.append(day)
        self.display()

    def clear(self):
        for key in self.buttons.keys():
            self.buttons[key][0].configure(bg=self.defaultcolor)

    def display(self):
        for r in range(7):
            if r == 0:
                ddate = calendar.datetime.date(self.year, self.month, 1) 
                prompt = ddate.strftime("%B %Y")
                label = tk.Button(self.root, text=prompt, width=10, borderwidth=0, command=self.clear)
                label.grid(row=r, column=2, columnspan=5, pady=10, sticky="w")
            elif r == 1:
                for c in range(7):
                    label = tk.Label(self.root, text=self.day_names[c])
                    label.grid(row=r, column=c)
            else:
                for c in range(7):
                    if len(self.days) > 0:
                        day = self.days.pop(0)
                    else:
                        day = "None"
                    if day == 0:
                        button = tk.Button(self.root, text='', borderwidth=0, state=tk.DISABLED)
                        ddate = None
                    else:
                        ddate = calendar.datetime.date(self.year, self.month, day)
                        button = tk.Button(self.root, text=day, borderwidth=0, command=lambda e=(r, c): self.click(e))
                    button.grid(row=r, column=c)
                    self.buttons["%s%s" % (r, c)] = (button, ddate)

        tk.Button(self.root, text="Ctrl-C", command=self.export).grid(row=r+1, column=3, columnspan=3, sticky="w", pady=10)
        tk.Button(self.root, text="<", command=self.previous).grid(row=r+1, column=0, pady=10)
        tk.Button(self.root, text=">", command=self.next).grid(row=r+1, column=6, pady=10)


if __name__ == "__main__":
    root = tk.Tk()
    root.title('Calendar')
    cal = MyCalendar(root)
    cal.display()
    ws = root.winfo_screenwidth()
    hs = root.winfo_screenheight()
    w = 295
    h = 250 + 20 # height + task bar
    x = ws - w*1.1
    y = hs - h*1.2
    root.geometry('+%d+%d' % (x, y))

    root.mainloop()
